@@ -1,31 +1,31 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* bus - The Input Bus
- * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2010 Red Hat, Inc.
+ * Copyright (C) 2008-2013 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2013 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
+ * USA
  */
 #ifndef __BUS_IBUS_IMPL_H_
 #define __BUS_IBUS_IMPL_H_
 
 #include <ibus.h>
 #include "connection.h"
+#include "component.h"
 #include "inputcontext.h"
-#include "registry.h"
 #include "factoryproxy.h"
 #include "panelproxy.h"
 #include "engineproxy.h"
@@ -52,8 +52,6 @@
     (bus_ibus_impl_get_default ())
 #define BUS_DEFAULT_KEYMAP \
     (bus_ibus_impl_get_keymap (BUS_DEFAULT_IBUS))
-#define BUS_DEFAULT_REGISTRY \
-    (bus_ibus_impl_get_registry (BUS_DEFAULT_IBUS))
 
 G_BEGIN_DECLS
 
@@ -75,7 +73,9 @@ BusIBusImpl     *bus_ibus_impl_get_default          (void);
 BusFactoryProxy *bus_ibus_impl_lookup_factory       (BusIBusImpl        *ibus,
                                                      const gchar        *path);
 IBusKeymap      *bus_ibus_impl_get_keymap           (BusIBusImpl        *ibus);
-BusRegistry     *bus_ibus_impl_get_registry         (BusIBusImpl        *ibus);
+BusComponent    *bus_ibus_impl_lookup_component_by_name
+                                                    (BusIBusImpl        *ibus,
+                                                     const gchar        *name);
 gboolean         bus_ibus_impl_is_use_sys_layout    (BusIBusImpl        *ibus);
 gboolean         bus_ibus_impl_is_embed_preedit_text
                                                     (BusIBusImpl        *ibus);
