@@ -238,7 +238,7 @@ find_emoji_data_list (IBusEmojiData *a,
         else if (strcmp_novariant (a_str, b->emoji, 0xfe0f, 0) == 0)
             return 0;
         else
-            return g_strcmp0 (a_str, b->emoji);
+            return -1;
         break;
     case EMOJI_NOVARIANT:
         if (strcmp_novariant (a_str, b->emoji, 0, 0xfe0e) == 0)
@@ -246,7 +246,7 @@ find_emoji_data_list (IBusEmojiData *a,
         else if (strcmp_novariant (a_str, b->emoji, 0, 0xfe0f) == 0)
             return 0;
         else
-            return g_strcmp0 (a_str, b->emoji);
+            return -1;
         break;
     default:;
     }
@@ -305,6 +305,7 @@ update_emoji_list (EmojiData *data,
                    gboolean   base_update)
 {
     GSList *list;
+    gboolean has_strict = FALSE;
     data->search_type = EMOJI_STRICT;
     list = g_slist_find_custom (
             data->list,
@@ -312,7 +313,7 @@ update_emoji_list (EmojiData *data,
             (GCompareFunc) find_emoji_data_list);
     if (list) {
         emoji_data_update_object (data, list->data);
-        return;
+        has_strict = TRUE;
     } else if (base_update) {
         emoji_data_new_object (data);
         return;
@@ -339,7 +340,8 @@ update_emoji_list (EmojiData *data,
             return;
         }
     }
-    emoji_data_new_object (data);
+    if (!has_strict)
+        emoji_data_new_object (data);
 }
 
 static void
