@@ -216,8 +216,12 @@ parse_compose_sequence (IBusComposeData *compose_data,
             compose_data->sequence[n] = codepoint;
         }
 
-        if (codepoint == IBUS_KEY_VoidSymbol)
-            g_warning ("Could not get code point of keysym %s", match);
+        if (codepoint == IBUS_KEY_VoidSymbol) {
+            g_warning ("Could not get code point of keysym %s: %s",
+                       match, line);
+            g_free (match);
+            goto fail;
+        }
         g_free (match);
         n++;
     }
@@ -378,7 +382,7 @@ ibus_compose_list_check_duplicated (GList *compose_list,
             
             if (n_outputs == unichar_length (output_chars)) {
                 int j = 0;
-                while (compose_data->values[j] && j < n_outputs) {
+                while (j < n_outputs && compose_data->values[j]) {
                     if (compose_data->values[j] != output_chars[j])
                         break;
                     ++j;
